/* wptKeyserverSearchDlg.cpp - Keyserver Searching
 *	Copyright (C) 2001-2006 Timo Schulz
 *
 * This file is part of WinPT.
 *
 * WinPT is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * WinPT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with WinPT; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <windows.h>
#include <stdio.h>

#include "resource.h"
#include "wptCommonCtl.h"
#include "wptKeyserver.h"
#include "wptErrors.h"
#include "wptTypes.h"
#include "wptGPG.h"
#include "wptContext.h" /* for passphrase_s */
#include "wptDlgs.h"
#include "wptNLS.h"


enum { KSS_COL_BITS = 0, KSS_COL_KEYID, KSS_COL_DATE, KSS_COL_UID };

int
hkp_recv_key2 (HWND dlg, const char *kserver, WORD port,
	       const char *pattern, int proto, char **r_fpr);


/* Initialize dialog. */
static listview_ctrl_t
on_init_dialog (HWND dlg, UINT msg, WPARAM wparam, LPARAM lparam)
{
    struct listview_column_s keysearch[] = {
        {0,  48, (char *)_("Size")},
        {1,  64, (char *)_("Key ID")},
        {2,  72, (char *)_("Creation")},
        {3, 256, (char *)_("User ID")},
        {0,   0, NULL}
    };
    char info[384];
    int i;
    listview_ctrl_t lv = NULL;
    
    SetDlgItemText (dlg, IDC_HKPSEARCH_RECV, _("&Receive"));
    SetDlgItemText (dlg, IDOK, _("&Cancel"));
    _snprintf (info, sizeof (info) - 1,
	_("Connect to '%s' to search for \"%s\""),
	((keyserver_ctx *)lparam)->name, ((keyserver_ctx *)lparam)->pattern);    
    SetDlgItemText (dlg, IDC_HKPSEARCH_INFO, info);
    SetWindowText (dlg, _("Keyserver Searching"));
    SetForegroundWindow (dlg);
    listview_new (&lv);
    lv->ctrl = GetDlgItem (dlg, IDC_HKPSEARCH_LIST);
    for (i = 0; keysearch[i].width; i++)
        listview_add_column (lv, &keysearch[i]);       
    listview_set_ext_style (lv);
    return lv;
}



static int
search_hkp_keys (HWND dlg, keyserver_ctx *ksc, listview_ctrl_t lv)
{
    int rc;
    int conn_fd = 0;
    char t[32];
    keyserver_key key;

    rc = kserver_search_begin (ksc->name, ksc->port, ksc->pattern, &conn_fd);
    if (rc) {
        msg_box (dlg, winpt_strerror (rc), _("Keyserver Search"), MB_ERR);
	return rc;
    }

    while (!kserver_search_next (conn_fd, &key)) {
	if (!key.bits)
	    continue;
	_snprintf (t, sizeof (t) - 1, "%d", key.bits);
	listview_add_item (lv, " ");
	listview_add_sub_item (lv, 0, KSS_COL_BITS, t);
	listview_add_sub_item (lv, 0, KSS_COL_KEYID, key.keyid);
	listview_add_sub_item (lv, 0, KSS_COL_DATE, key.date);
	listview_add_sub_item (lv, 0, KSS_COL_UID, key.uid);
    }

    kserver_search_end (conn_fd);
    return 0;
}


/* Fetch all selected keys from the keyserver in @ksc. */
static void
fetch_keys (HWND dlg, listview_ctrl_t lv, keyserver_ctx *ksc)
{
    char keyid[32];
    char *fpr;
    int n, rc = 0;

    n = listview_count_items (lv, 1);
    if (!n) {
	msg_box (dlg, _("Please select a key."), _("Keyserver Search"), MB_ERR);
	return;
    }

    for (n=0; n < listview_count_items (lv, 0); n++) {
	if (!listview_get_item_state (lv, n))
	    continue;

	listview_get_item_text (lv, n, KSS_COL_KEYID, keyid, sizeof (keyid)-1);
	rc = hkp_recv_key2 (dlg, ksc->name, ksc->port, keyid, 0, &fpr);
	if (!rc && fpr != NULL) {
	    keycache_update (0, fpr);
	    free_if_alloc (fpr);
	}
    }
}


/* Dialog box procedure for keyserver searching. */
BOOL CALLBACK
hkpsearch_dlg_proc (HWND dlg, UINT msg, WPARAM wparam, LPARAM lparam)
{
    static listview_ctrl_t lv = NULL;
    static keyserver_ctx *ksc;
    int rc = 0;
    
    switch (msg) {
    case WM_INITDIALOG:
        ksc = (keyserver_ctx *)lparam;
        if (!ksc)
            dlg_fatal_error (dlg, "Could not get dialog param.");
	lv = on_init_dialog (dlg, msg, wparam, lparam);
	rc = search_hkp_keys (dlg, ksc, lv);
	if (rc) {
	    EndDialog (dlg, FALSE);
	    return FALSE;
	}
        return TRUE;
        
    case WM_DESTROY:
        if (lv) {
            listview_release (lv);
            lv = NULL;
        }
        return FALSE;
        
    case WM_SYSCOMMAND:
        if (LOWORD (wparam) == SC_CLOSE)
            EndDialog (dlg, TRUE);
        return FALSE;
        
    case WM_COMMAND:
        switch (LOWORD (wparam)) {
        case IDOK:
            EndDialog (dlg, TRUE);
            return TRUE;
            
        case IDCANCEL:
            EndDialog (dlg, FALSE);
            return FALSE;
            
        case IDC_HKPSEARCH_RECV:
	    fetch_keys (dlg, lv, ksc);
            return TRUE;
        }
        break;
    }
    
    return FALSE;
}
